# -*- coding: utf-8 -*-
'''
Copyright 2012 Rodrigo Pinheiro Matias <rodrigopmatias@gmail.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

from hwm.plugin.base import BaseColectorPlugin, BaseGraphicPlugin, font_title
from hwm.humanize import bytes

try:
    from PIL import ImageColor
except ImportError:
    import ImageColor

import subprocess
import re
import os

class Disk(BaseColectorPlugin, BaseGraphicPlugin):

    def _title(self, canvas):
        title = getattr(self._params, 'title', 'Disk usage for %s' % self._params.mount_point)
        width, height = canvas.textsize(title, **{'font': font_title})

        x = int(canvas.im.size[0] / 2.0 - width / 2.0)
        y = 10

        canvas.text((x, y), title, **{
            'fill': ImageColor.getcolor('#000', 'RGB'),
            'font': font_title
        })

    def __init__(self, *args, **kargs):
        BaseColectorPlugin.__init__(self, *args, **kargs)
        self._period = 60 * 5

    @property
    def params(self):
        return {
            'mount_point': self._params.mount_point,
            'size': 60
        }

    @property
    def filename(self):
        return 'disk.png'

    @property
    def colors(self):
        return ('bbbbbb', '0000ff', '007700')

    def humanize(self, number):
        return bytes(number)
    
    @classmethod
    def description(class_):
        dct = BaseColectorPlugin.description()
        dct.update({
            'name': 'Disk collector',
            'slug': 'disk',
            'options': [
                (
                    '--mount-point',
                    {
                        'help': 'Specific the mount point',
                        'dest': 'mount_point'
                    }
                )
            ]
        })

        return dct

    def _prepare(self, **kargs):
        dbc = self.get_connector()
        cursor = dbc.cursor()

        try:
            cursor.execute(
                '''
CREATE TABLE IF NOT EXISTS storage(
    ID INTEGER PRIMARY KEY AUTOINCREMENT,
    MOUNT_POINT VARCHAR(300) NOT NULL,
    SIZE INTEGER NOT NULL DEFAULT 0,
    USED INTEGER NOT NULL DEFAULT 0,
    AVAIL INTEGER NOT NULL DEFAULT 0,
    COLECT_MOMENT TIMESTAMP DEFAULT CURRENT_TIMESTAMP
)
                '''
            )
        except Exception, e:
            print e
        else:
            try:
                cursor.execute(
                    '''
INSERT INTO storage(MOUNT_POINT, SIZE, USED, AVAIL) VALUES(:mount_point, :size, :used, :avail)
                    ''',
                    kargs
                )
            except Exception, e:
                print e
            else:
                dbc.commit()

        cursor.close()
        dbc.close()

    # def do_colect(self):
    #     for line in subprocess.Popen(['/bin/df'], stdout=subprocess.PIPE).stdout:
    #         rst = re.match('^.+\s+(?P<size>[0-9]+)\s+(?P<used>[0-9]+)\s+(?P<avail>[0-9]+)\s+(?P<pct>[0-9]+)%\s+(?P<mount_point>(/([a-z]|0-9)+)+)$', line)
    #         if rst is not None and rst.groupdict().get('mount_point') == self._params.mount_point:
    #             self._prepare(**rst.groupdict())
    #             break

    def do_colect(self):
        st = os.statvfs(self._params.mount_point)

        colect = {
            'mount_point': self._params.mount_point,
            'size': (st.f_blocks * st.f_frsize),
            'avail': (st.f_bavail * st.f_frsize)
        }

        colect.update({
            'used': colect.get('size') - colect.get('avail'),
            'pct': round(float(colect.get('size') - colect.get('avail')) / float(colect.get('size')) * 100, 2)
        })

        self._prepare(**colect)

    def clear(self):
        dbc = self.get_connector()
        cursor = dbc.cursor()

        try:
            cursor.execute(
                'DELETE FROM storage WHERE mount_point = :mount_point', 
                self._params.__dict__
            )
        except Exception, e:
            print 'Ocorreu um erro executando a limpeza.'
            print e
        else:
            dbc.commit()

    def do_extract(self, mount_point, size = 10):
        dbc = self.get_connector()
        cursor = dbc.cursor()
        sql = '''
SELECT
    size,
    used,
    avail
FROM
    storage
WHERE
    mount_point = :mount_point
ORDER BY
    id DESC
LIMIT
    :size
'''
        
        count = 0
        last = 0
        for row in dbc.execute(sql, vars()):
            yield row
            last = row[0]
            count += 1

        while count < size:
            yield (last, 0, 0)
            count += 1

    def get_column(self):
        return ('size', 'used', 'avail')